@@ -56,6 +56,10 @@ class Service(object):
         self.path = path
         self.port = port
         self.cli_name = service_name
+        if not hasattr(self, 'metadata'):
+            # metadata is an option thing that comes from .extra.json
+            # so if it's not there we just default to an empty dict.
+            self.metadata = {}
 
     def _create_operation_objects(self):
         logger.debug("Creating operation objects for: %s", self)
@@ -78,7 +82,7 @@ class Service(object):
 
     @property
     def region_names(self):
-        return self.metadata['regions'].keys()
+        return self.metadata.get('regions', {}).keys()
 
     def _build_endpoint_url(self, host, is_secure):
         if is_secure:
@@ -106,23 +110,55 @@ class Service(object):
 
         :type endpoint_url: str
         :param endpoint_url: You can explicitly override the default
-            computed endpoint name with this parameter.
+            computed endpoint name with this parameter.  If this arg is
+            provided then neither ``region_name`` nor ``is_secure``
+            is used in building the final ``endpoint_url``.
+            ``region_name`` can still be useful for services that require
+            a region name independent of the endpoint_url (for example services
+            that use Signature Version 4, which require a region name for
+            use in the signature calculation).
+
         """
-        if not region_name:
+        if region_name is None:
             region_name = self.session.get_variable('region')
-            if region_name is None and not self.global_endpoint:
-                envvar_name = self.session.env_vars['region'][1]
-                raise NoRegionError(env_var=envvar_name)
-        if region_name not in self.metadata['regions']:
+        if endpoint_url is not None:
+            # Before getting into any of the region/endpoint
+            # logic, if an endpoint_url is explicitly
+            # provided, just use what's been explicitly passed in.
+            return get_endpoint(self, region_name, endpoint_url)
+        if region_name is None and not self.global_endpoint:
+            # The only time it's ok to *not* provide a region is
+            # if the service is a global_endpoint (e.g. IAM).
+            envvar_name = self.session.env_vars['region'][1]
+            raise NoRegionError(env_var=envvar_name)
+        if region_name not in self.region_names:
             if self.global_endpoint:
+                # If we haven't provided a region_name and this is a global
+                # endpoint, we can just use the global_endpoint (which is a
+                # string of the hostname of the global endpoint) to construct
+                # the full endpoint_url.
                 endpoint_url = self._build_endpoint_url(self.global_endpoint,
                                                         is_secure)
                 region_name = 'us-east-1'
             else:
+                # Otherwise we've specified a region name that is
+                # not supported by the service so we raise
+                # an exception.
                 raise ServiceNotInRegionError(service_name=self.endpoint_prefix,
                                               region_name=region_name)
+        # The 'regions' dict can call out the specific hostname
+        # to use for a particular region.  If this is the case,
+        # this will have precedence.
+        # TODO: It looks like the region_name overrides shouldn't have the
+        # protocol prefix.  Otherwise, it doesn't seem possible to override
+        # the hostname for a region *and* support both http/https.  Should
+        # be an easy change but it will be backwards incompatible to anyone
+        # creating their own service descriptions with region overrides.
         endpoint_url = endpoint_url or self.metadata['regions'][region_name]
         if endpoint_url is None:
+            # If the entry in the 'regions' dict is None,
+            # then we fall back to the patter of
+            # endpoint_prefix.region.amazonaws.com.
             host = '%s.%s.amazonaws.com' % (self.endpoint_prefix, region_name)
             endpoint_url = self._build_endpoint_url(host, is_secure)
         return get_endpoint(self, region_name, endpoint_url)
